﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading;
using RocketAd;
using RocketGM;
using RocketLog;
using UnityEngine;

namespace RocketRevenueTracker
{
    public class RocketRevenueTrackerController : MonoBehaviour
    {
        [SerializeField]
        private RevenueInfoDTO[] _revenuesToBeTracked = new RevenueInfoDTO[] 
        {
            new RevenueInfoDTO(0.03f, "_03"),
            new RevenueInfoDTO(0.05f, "_05"),
            new RevenueInfoDTO(0.10f, "_10"),
            new RevenueInfoDTO(0.20f, "_20"),
            new RevenueInfoDTO(0.50f, "_50"),
            new RevenueInfoDTO(1.00f, "_100")
        };

        private List<RocketRevenueTracker> _revenueTrackers = new List<RocketRevenueTracker>();

        [SerializeField] private int _revenueFlushIntervalInSeconds;

        private List<double> _trackedRevenues;
        private WaitForSeconds _waitForSeconds;

#if ROC_GM_APPSFLYER
        [SerializeField] private bool _sendAppsflyer = true;
#endif
#if ROC_FACEBOOK
        [SerializeField] private bool _sendFacebook = true;
#endif
#if ROC_FIREBASE
        [SerializeField] private bool _sendFirebase = true;
#endif

        private static int _unityThreadId = -1;

        private RocLog Log = new RocLog(nameof(RocketRevenueTrackerController), DebugLevels.Debug);

        private bool _isLocked = false;

        private void Awake()
        {
            DontDestroyOnLoad(gameObject);
        }

        private void Start()
        {
            _revenueTrackers = new List<RocketRevenueTracker>();

            foreach (var revenueInfo in _revenuesToBeTracked)
            {
                _revenueTrackers.Add(new RocketRevenueTracker(revenueInfo.Threshold, revenueInfo.Suffix, true));
            }

            _trackedRevenues ??= new List<double>();
            _waitForSeconds = new WaitForSeconds(_revenueFlushIntervalInSeconds);

            _unityThreadId = Thread.CurrentThread.ManagedThreadId;
            Log.Debug($"Initializing RocGMRevenueTracker.\nThread ID: {_unityThreadId}");
            
            Log.Debug($"{_revenueTrackers.Count} tracker found.");

#if ROC_GM_APPSFLYER
            RocketRevenueTracker.SendAppsflyer = _sendAppsflyer;
#endif
#if ROC_FACEBOOK
            RocketRevenueTracker.SendFacebook = _sendFacebook;
#endif
#if ROC_FIREBASE
            RocketRevenueTracker.SendFirebase = _sendFirebase;
#endif

            RocGm.DoAfterInitialized(() =>
            {
                foreach (var revenueTracker in _revenueTrackers)
                {
                    revenueTracker.TrySendRevenue(revenueTracker.RevenueAmount);
                }

                StartCoroutine(RevenueFlushEnumerator());
            });
        }

        private void OnEnable()
        {
            RocAd.ImpressionTracked += OnImpressionTracked;
        }

        private void OnDisable()
        {
            RocAd.ImpressionTracked -= OnImpressionTracked;
        }

        private void OnApplicationFocus(bool hasFocus)
        {
            if (!hasFocus)
            {
                FlushCachedRevenues();
            }
        }

        private IEnumerator RevenueFlushEnumerator()
        {
            while (gameObject.activeSelf)
            {
                yield return _waitForSeconds;
                FlushCachedRevenues();
            }
        }

        private void FlushCachedRevenues()
        {
            if (_trackedRevenues == null)
                return;

            if (_revenueTrackers == null)
                return;

            if (_isLocked)
                return;

            _isLocked = true;

            Log.Debug($"Flushing cached revenues to the related SDKs, sending {_trackedRevenues.Count} revenues.");

            foreach (var revenueTracker in _revenueTrackers)
            {
                if (revenueTracker == null)
                    continue;

                foreach (var trackedRevenue in _trackedRevenues)
                {
                    revenueTracker.RevenueAmount += (float)trackedRevenue;
                    revenueTracker.TrySendRevenue(revenueTracker.RevenueAmount);
                }
            }

            // Clear the tracked revenue list.
            _trackedRevenues.Clear();

            _isLocked = false;
        }

        private void OnImpressionTracked(RocImpressionData rocImpressionData)
        {
            Log.Debug($"Tracked/cached revenue for revenue tracker, the revenue was: {rocImpressionData.Revenue}");

            if (_isLocked)
            {
                StartCoroutine(AddRevenueWhenAvailableRoutine(rocImpressionData.Revenue));
                return;
            }

            _trackedRevenues ??= new List<double>();
            _trackedRevenues.Add(rocImpressionData.Revenue);
        }

        private IEnumerator AddRevenueWhenAvailableRoutine(double revenue)
        {
            var addRevenueWhenAvailableRoutine = new WaitUntil(IsReady);
            yield return addRevenueWhenAvailableRoutine;


            _trackedRevenues ??= new List<double>();
            _trackedRevenues.Add(revenue);


            bool IsReady() => !_isLocked;
        }
    }
}